﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace Memento.Common.WaitControl
{
    /// <summary>
    /// 等待进度条控件
    /// </summary>
    public partial class WaitProgress : UserControl
    {
        #region 成员变量

        private Window m_Window;
        private UIElementCollection m_RootControls;
        private bool m_AutoCloseAfterFinish;

        #endregion

        #region 构造函数

        /// <summary>
        /// 弹窗等待进度条，注意，此时如果要在异步线程中关闭等待窗口，则调用 Show 方法需要放在异步线程启动之后。
        /// </summary>
        /// <param name="parent">依赖的父窗口</param>
        /// <param name="isIndeterminate">设置显示实际值，还是显示一般的连续进度反馈。</param>
        /// <param name="autoCloseAfterFinish">结束之后是否自动关闭</param>
        public WaitProgress(Window parent, bool isIndeterminate = false, bool autoCloseAfterFinish = false)
        {
            InitializeComponent();

            try
            {
                MouseRightButtonDown += WaitProgress_MouseRightButtonDown;
                m_RootControls = null;
                m_AutoCloseAfterFinish = autoCloseAfterFinish;
                pbProgressBar.IsIndeterminate = isIndeterminate;

                m_Window = new Window();
                m_Window.WindowStyle = WindowStyle.None;
                m_Window.Background = new SolidColorBrush(Colors.Transparent);
                m_Window.AllowsTransparency = true;
                m_Window.Owner = parent;
                m_Window.WindowStartupLocation = WindowStartupLocation.CenterOwner;
                if (parent == null)
                {
                    m_Window.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                }
                m_Window.SizeToContent = SizeToContent.WidthAndHeight;
                m_Window.ShowInTaskbar = false;
                m_Window.Content = this;

                bContainer.BorderThickness = new Thickness(1);
                bContainer.Background = new SolidColorBrush(Colors.GhostWhite);
            }
            catch (Exception ex)
            {
                MessageBox.Show("初始化 WaitProgress 控件异常，详细信息：\r\n" + ex.Message);
            }
        }

        /// <summary>
        /// 附在已有控件上的等待进度条
        /// </summary>
        /// <param name="rootControl">依赖的根控件</param>
        /// <param name="isIndeterminate">设置显示实际值，还是显示一般的连续进度反馈。</param>
        /// <param name="autoCloseAfterFinish">结束之后是否自动关闭</param>
        public WaitProgress(UIElementCollection rootControl, bool isIndeterminate = false, bool autoCloseAfterFinish = false, bool fullGrid = false)
        {
            InitializeComponent();

            try
            {
                MouseRightButtonDown += WaitProgress_MouseRightButtonDown;

                m_Window = null;
                m_RootControls = rootControl;
                m_AutoCloseAfterFinish = autoCloseAfterFinish;
                pbProgressBar.IsIndeterminate = isIndeterminate;

                bContainer.BorderThickness = new Thickness(0);
                bContainer.Background = new SolidColorBrush(Colors.Transparent);

                if (fullGrid)
                {
                    Grid.SetRowSpan(this, 100);
                    Grid.SetColumnSpan(this, 100);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("初始化 WaitProgress 控件异常，详细信息：\r\n" + ex.Message);
            }
        }

        #endregion

        #region 事件处理

        // 禁用鼠标右键菜单等其他操作
        private void WaitProgress_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
        }

        #endregion

        #region 公开方法

        /// <summary>
        /// 设置当前进度和提示信息
        /// </summary>
        /// <param name="percent">当前进度</param>
        /// <param name="tipContent">当前提示信息</param>
        public void SetProgress(double percent, string tipContent = "")
        {
            if (percent >= 0.0 && percent <= 100.0)
            {
                lbTip.Dispatcher.BeginInvoke((Action)delegate ()
                {
                    if (!string.IsNullOrEmpty(tipContent))
                    {
                        lbTip.Content = tipContent;
                    }
                    pbProgressBar.Value = percent;
                });
            }
            if (m_AutoCloseAfterFinish && percent == 100.0)
            {
                Dispatcher.BeginInvoke((Action)delegate ()
                {
                    Close();
                });
            }
        }

        /// <summary>
        /// 显示等待进度条
        /// </summary>
        /// <param name="tipContent">进度信息</param>
        public void Show(string tipContent = "请稍候...")
        {
            lbTip.Content = tipContent;
            if (m_Window != null)
            {
                m_Window.ShowDialog();
            }
            else if (m_RootControls != null)
            {
                if (!m_RootControls.Contains(this))
                {
                    m_RootControls.Add(this);
                }
            }
        }

        /// <summary>
        /// 关闭等待进度条
        /// </summary>
        public void Close()
        {
            if (m_Window != null)
            {
                m_Window.Dispatcher.BeginInvoke((Action)delegate ()
                {
                    m_Window.Close();
                    m_Window.Content = null;
                });
            }
            else if (m_RootControls != null)
            {
                Dispatcher.BeginInvoke((Action)delegate ()
                {
                    m_RootControls.Remove(this);
                });
            }

            GC.Collect();
            GC.WaitForFullGCComplete();
        }

        #endregion
    }
}